<?php

namespace AltSolution\Admin\Cron;

use AltSolution\Admin\Models\Cron;
use AltSolution\Admin\Models\CronLog;
use Carbon\Carbon;
use Illuminate\Support\Facades\Artisan;
use Symfony\Component\Console\Command\Command;

class Repository implements RepositoryInterface
{
    public function getAll()
    {
        return cache()->rememberForever('cms_cron', function () {
            try {
                return Cron::query()
                    ->where('is_enabled', true)
                    ->get();
            } catch (\PDOException $ex) {
                // if no database connection available assume no jobs
                return collect([]);
            }
        });
    }

    public function cleanCache()
    {
        cache()->forget('cms_cron');
    }

    public function addLog(Cron $cron, Carbon $start, Carbon $end, $output)
    {
        $entry = new CronLog;
        $entry->fill([
            'cron_id' => $cron->id,
            'start_at' => $start,
            'end_at' => $end,
            // TODO: not possible receive at this time
            'exit_code' => 0,
            'output' => $output,
        ]);
        $entry->save();

        $lottery = config('admin.cron_lottery');
        if (random_int(1, $lottery[1]) <= $lottery[0]) {
            $this->truncLog($cron);
        }
    }

    public function truncLog(Cron $cron)
    {
        $minId = CronLog::query()
            ->where('cron_id', $cron->id)
            ->orderBy('start_at', 'desc')
            ->limit($historyLength = 100)
            ->get()
            ->min('id');

        CronLog::query()
            ->where('cron_id', $cron->id)
            ->where('id', '<', $minId)
            ->delete();
    }

    public function flushLog()
    {
        CronLog::query()->truncate();
    }

    public function getCommands()
    {
        $filter = [
            'make:*',
            'migrate:*',
        ];
        $whitelist = false;

        $allCommands = collect(Artisan::all());

        if (!empty($filter)) {
            $allCommands = $allCommands->filter(function (Command $command) use ($filter, $whitelist) {
                foreach ($filter as $filter) {
                    if (fnmatch($filter, $command->getName())) {
                        return $whitelist;
                    }
                }
                return !$whitelist;
            });
        }

        /*
        return $all_commands->sortBy(function (Command $command) {
            return $command->getDescription();
        });
        */

        return $allCommands;
    }
}
