<?php

namespace AltSolution\Admin\Form\Field;

use AltSolution\Admin\Form\AbstractField;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Model;

class Select extends AbstractField
{
    use ChoicesTrait;

    protected $view = 'select';

    protected function customViewData()
    {
        $choices = $this->getChoices();

        $nullable = $this->getOption('nullable', false);
        if ($nullable) {
            $nullTitle = $this->getOption('placeholder');
            $choices = [null => $nullTitle] + $choices;
        }

        $multiple = $this->getOption('multiple', false);

        return compact('choices', 'multiple');
    }

    public function normalizeValue($value)
    {
        // for eloquent relations
        if ($this->getOption('multiple')) {
            // belongsToMany() case
            if ($value instanceof Collection) {
                return $value->modelKeys();
            }
        } else {
            // belongsTo() case
            if ($value instanceof Model) {
                return $value->getKey();
            }
        }
        return $value;
    }

    public function getValidationName()
    {
        $name = $this->getName();
        // array validation
        if ($this->getOption('multiple')) {
            $name .= '.*';
        }

        return $name;
    }
}
