<?php

namespace AltSolution\Admin\Forms;

use AltSolution\Admin\Form;
use AltSolution\Admin\Models\AclRole;
use AltSolution\Admin\Modules\User\UserFormInterface;

class UserForm extends Form\AbstractFactory implements UserFormInterface
{
    public function buildForm(Form\BuilderInterface $builder)
    {
        $itemId = $builder->getData('id');

        $builder->form([
            'method' => 'post',
            'action' => route('admin::user_save'),
            'enctype' => 'multipart/form-data',
        ]);

        $builder->hidden('id');
        $builder->selectModel('acl_role_id', [
            'label' => trans('admin::user.acl_role'),
            'model' => AclRole::class,
            'title_key' => 'description',
            'required' => true,
            'rules' => 'numeric|exists:acl_roles,id',
        ]);
        $builder->image('avatar', [
            'label' => trans('admin::user.avatar'),
            'help' => trans('admin::user.avatar_description'),
            'rules' => 'max:2000|dimensions:min_width=128,min_height=128,max_width=3000,max_height=3000',
        ]);
        $builder->text('name', [
            'label' => trans('admin::user.username'),
            'required' => true,
            'rules' => 'max:100',
        ]);
        $builder->email('email', [
            'label' => trans('admin::user.email'),
            'required' => true,
            'rules' => 'unique:users,email,' . $itemId,
        ]);
        $builder->password('password', [
            'label' => trans('admin::user.password'),
            'different' => 'name',
            'required' => 'unless:id',
            'rules' => [
                'confirmed',
                'min:6',
                'max:60',
                'regex:/[a-z]/', // letter lower
                'regex:/[A-Z]/', // letter upper
                'regex:/[0-9]/', // number
                'regex:/[!@#$%^&*?()\-_=+{};:,<.>]/' // special
            ],
        ]);
        $builder->password('password_confirmation', [
            'label' => trans('admin::user.confirm_password'),
            'identical' => 'password',
            'required' => 'unless:id',
        ]);
        $user = auth()->user();
        $builder->checkbox('is_active', [
            'label' => trans('admin::user.activate_question'),
            'placeholder' => trans('admin::common.activate'),
            'help' => trans('admin::user.activate_description'),
            // disable for current user
            'disabled' => 'if:id,' . $user->id,
        ]);
    }
}