<?php

namespace AltSolution\Admin\Http\Controllers;

use AltSolution\Admin\Modules\Menu\MenuFormInterface;
use AltSolution\Admin\Modules\Menu\MenuItemModelInterface;
use AltSolution\Admin\Modules\Menu\MenuModelInterface;
use Illuminate\Http\Request;

class MenuController extends Controller
{
    public function getIndex()
    {
        $this->authorize('permission', 'menu.list');

        $items = app(MenuModelInterface::class)->query()
            ->paginate(config('admin.items_per_page', 20));

        $this->layout
            ->setMenuSection('menu')
            ->setTitle(trans('admin::menu.title'));
        return view('admin::menu.list', compact('items'));
    }

    public function getEdit($id = null)
    {
        $this->authorize('permission', 'menu.edit');

        $menu = null;
        if ($id) {
            $menu = app(MenuModelInterface::class)->query()->find($id);
        }

        $form = cms_create_form(MenuFormInterface::class, $menu);

        $this->layout
            ->setMenuSection('menu')
            ->setTitle(trans($id ? 'admin::menu.edit' : 'admin::menu.add'))
            ->addBreadcrumb(trans('admin::menu.title'), route('admin::menu_list'));
        return view('admin::menu.edit', compact('form'));
    }

    public function postSave(Request $request)
    {
        $this->authorize('permission', 'menu.edit');

        $itemId = $request->input('id');
        $menu = app(MenuModelInterface::class)->query()->firstOrNew(['id' => $itemId]);
        $form = cms_create_form(MenuFormInterface::class, $menu);
        $validData = $form->validate($request);

        $menu->fill($validData);
        event('admin.model.menu.saving', $menu);
        $menu->save();
        event('admin.model.menu.saved', $menu);

        $this->layout->addNotify('success', trans('admin::menu.saved'));

        if ($request->button_apply) {
            return redirect()->route('admin::menu_edit', $menu->id);
        }

        return redirect()->route('admin::menu_list');
    }

    public function action(Request $request)
    {
        $action = $request->input('action');
        $ids = $request->input('ids');
        if (!$ids) {
            return;
        }

        if ($action == 'delete') {
            $this->authorize('permission', 'menu.delete');

            foreach ($ids as $id) {
                $menu = app(MenuModelInterface::class)->query()->find($id);
                if (!$menu) {
                    continue;
                }

                event('admin.model.menu.deleting', $menu);
                $items = app(MenuItemModelInterface::class)->query()->where('menu_id', $menu->id)->get();
                foreach ($items as $item) {
                    $item->delete();
                }
                $menu->delete();
                event('admin.model.menu.deleted', $menu);
            }
        }
    }
}
