<?php

namespace AltSolution\Admin\Option;

use AltSolution\Admin\Models\Option;
use Illuminate\Support\Arr;

class Repository implements RepositoryInterface
{
    /**
     * @var array|null
     */
    private $options = null;

    /**
     * @return array
     */
    private function getOptions()
    {
        if ($this->options === null) {
            $this->options = cache()->remember('cms_options', config('admin.option_cache_ttl'), function () {
                return Option::all()
                    ->pluck('value', 'name')
                    ->all();
            });
        }

        return $this->options;
    }

    public function get($name, $default = null)
    {
        $options = $this->getOptions();

        return Arr::get($options, $name, $default);
    }

    public function getMany(array $names, $default = null)
    {
        $options = $this->getOptions();

        $result = [];
        foreach ($names as $name) {
            $result[$name] = Arr::get($options, $name, $default);
        }

        return $result;
    }

    public function getAll()
    {
        $options = $this->getOptions();

        return $options;
    }

    public function set($name, $value)
    {
        /** @var Option $option */
        $option = Option::query()->firstOrNew(compact('name'));
        $option->value = (string)$value;
        $option->save();

        cache()->forget('cms_options');
        $this->options = null;
    }

    public function setMany(array $options)
    {
        foreach ($options as $name => $value) {
            /** @var Option $option */
            $option = Option::query()->firstOrNew(compact('name'));
            $option->value = (string)$value;
            $option->save();
        }

        cache()->forget('cms_options');
        $this->options = null;
    }
}