<?php

namespace AltSolution\Admin\System;

trait CounterTrait
{
    /**
     * @var integer
     */
    protected $counter = null;
    /**
     * @var callable
     */
    protected $counterCallback = null;

    /**
     * @return int
     */
    public function getCounter()
    {
        if ($this->counter === null) {
            if ($this->counterCallback !== null) {
                $counter = call_user_func($this->counterCallback);
                if (!is_numeric($counter)) {
                    throw new \RuntimeException('Invalid counter return value');
                }
            } else {
                $counter = 0;
                if ($this instanceof \Traversable) {
                    /** @var CounterTrait $child */
                    foreach ($this as $child) {
                        $counter += $child->getCounter();
                    }
                }
            }
            $this->counter = $counter;
        }

        return $this->counter;
    }

    /**
     * @return bool
     */
    public function hasCounter()
    {
        $counter = $this->getCounter();

        return $counter > 0;
    }

    public function setCounterCallback(callable $callback)
    {
        $this->counterCallback = $callback;

        return $this;
    }
}

