<?php

namespace AltSolution\Admin\Twig\Extension;

use AltSolution\Admin\Form\AbstractComponent;
use AltSolution\Admin\Form\Form;
use Illuminate\Support\Str;
use Twig\Extension\AbstractExtension;
use Twig\TwigFilter;
use Twig\TwigFunction;

class FormExtension extends AbstractExtension
{
    public function getName()
    {
        return 'AltSolution_Admin_Form';
    }

    public function getFunctions()
    {
        return [
            new TwigFunction('form_field', [$this, 'form_field'], ['is_safe' => ['html']]),

            new TwigFunction('form_open', [$this, 'form_open'], ['is_safe' => ['html']]),
            new TwigFunction('form_close', [$this, 'form_close'], ['is_safe' => ['html']]),
            new TwigFunction('form_submit', [$this, 'form_submit'], ['is_safe' => ['html']]),

            new TwigFunction('html_attributes', [$this, 'html_attributes'], ['is_safe' => ['html']]),
            new TwigFunction('html_id', [$this, 'html_id'], ['is_safe' => ['html']]),
        ];
    }

    public function getFilters()
    {
        return [
            new TwigFilter('form_custom', [$this, 'form_custom'], ['is_safe' => ['html']]),

            new TwigFilter('html_inline', [$this, 'html_inline'], ['is_safe' => ['html']]),
        ];
    }

    public function form_field()
    {
        $args = func_get_args();
        $form = array_shift($args);
        return call_user_func_array([$form, 'renderComponent'], $args);
    }

    public function form_custom()
    {
        $args = func_get_args();

        $content = array_shift($args);
        /** @var Form $form */
        $form = array_shift($args);
        /** @var AbstractComponent $field */
        $field = $form->get($args[0]);
        $field->setOption('content', $content);

        return call_user_func_array([$form, 'renderComponent'], $args);
    }

    public function form_open(Form $form, array $options = [])
    {
        return $form->renderComponent('form_open', $options);
    }

    public function form_close(Form $form)
    {
        return $form->renderComponent('form_close');
    }

    public function form_submit(Form $form, array $options = [])
    {
        return $form->renderComponent('form_submit', $options);
    }

    public function html_inline($value)
    {
        // remove multiple spaces, newlines
        // $value = preg_replace('~\n+~u', '', $value);
        $value = preg_replace('~\s+~u', ' ', $value);

        return trim($value);
    }

    public function html_attributes(array $attributes)
    {
        $html = [];
        foreach ($attributes as $key => $value) {
            $element = $this->attribute($key, $value);
            if (! is_null($element)) {
                $html[] = $element;
            }
        }
        return count($html) > 0 ? ' ' . implode(' ', $html) : '';
    }

    public function html_id()
    {
        return 'rnd-' . Str::random();
    }

    protected function attribute($key, $value)
    {
        // required, readonly, etc.
        if (is_numeric($key)) {
            return $value;
        }
        // Treat boolean attributes as HTML properties
        if (is_bool($value) && $key != 'value') {
            return $value ? $key : '';
        }
        if (! is_null($value)) {
            return $key . '="' . e($value) . '"';
        }
    }

}
