<?php

namespace AltSolution\Admin\Upload;

use Exception;
use Illuminate\Http\Request;
use Illuminate\Http\UploadedFile;
use SplFileInfo;

interface UploadsInterface
{
    /**
     * Save upload in storage
     * @param string $property Upload field
     * @param UploadedFile $file
     * @throws Exception
     */
    function uploadSave($property, UploadedFile $file);

    /**
     * Save upload in storage from file object
     * @param string $property Upload field
     * @param SplFileInfo $file
     * @throws Exception
     */
    function uploadSaveFile($property, SplFileInfo $file);

    /**
     * Save upload in storage from url
     * @param string $property
     * @param string $url
     * @throws Exception
     */
    function uploadSaveUrl($property, $url);

    /**
     * @param string $property
     * @param string $filename
     * @param string $content
     */
    function uploadSaveContent($property, $filename, $content);

    /**
     * Automatic save and delete uploads from storage
     * @param Request $request
     */
    function uploadAllSave(Request $request);

    /**
     * Get upload url
     * @param string $property Upload field
     * @return null|string
     */
    function uploadUrl($property);

    /**
     * Get upload file
     * @param string $property Upload field
     * @return null|string
     */
    function uploadFile($property);

    /**
     * Delete upload from storage
     * @param string $property Upload field
     */
    function uploadDelete($property);

    /**
     * Delete all uploads from storage
     */
    function uploadAllDelete();
}

