<?php

use AltSolution\Admin\Form;
use AltSolution\Admin\EmailTemplate;
use AltSolution\Admin\Option;
use AltSolution\Admin\Seo;
use AltSolution\Admin\System;
use AltSolution\Admin\Upload;
use AltSolution\Admin\Image;
use Illuminate\Http\UploadedFile;
use Psr\Log\LoggerInterface;

/*
 * Helpers naming:
 * . one action: ns-verb-noun
 * . . cms_send_template()
 * . many actions: ns-noun
 * . . cms_seo()
 *
 */

/**
 * @return System\System
 */
function cms_system()
{
    return app(System\System::class);
}

/**
 * @return string
 */
function cms_admin_url()
{
    return config('admin.admin_url', 'admin');
}

function cms_admin_asset($path)
{
    // TODO: dist/ ???
    $ver = app()->environment() === 'production' ? System\System::VERSION : uniqid();
    return app('url')->asset('vendor/admin/dist/' . $path . '?' . http_build_query(compact('ver')));
}

/**
 * TODO: invalid naming
 * @return array
 */
function cms_locales()
{
    return array_keys(config('app.locales'));
}

/**
 * @return string
 */
function cms_admin_locale()
{
    $locale = config('admin.locale');
    if ($locale !== null) {
        return $locale;
    }

    return config('app.locale');
}

/**
 * @return string
 */
function cms_default_locale()
{
    $locale = config('app.default_locale');
    if ($locale !== null) {
        return $locale;
    }

    return config('app.locale');
}

/**
 * TODO: invalid naming
 * @param string $name
 * @param mixed $default
 * @return string
 */
function cms_option($name, $default = null)
{
    /** @var Option\RepositoryInterface $repository */
    $repository = app(Option\RepositoryInterface::class);

    return $repository->get($name, $default);
}

/**
 * TODO: invalid naming
 * @return array
 */
function cms_options()
{
    /** @var Option\RepositoryInterface $repository */
    $repository = app(Option\RepositoryInterface::class);

    return $repository->getAll();
}

/**
 * Construct form in runtime
 * @param callable $callback
 * @param mixed $data
 * @return Form\Form
 */
function cms_construct_form(callable $callback, $data = null)
{
    $builder = app(Form\BuilderInterface::class);
    if ($data !== null) {
        $builder->setDataSource($data);
    }
    $fields = call_user_func($callback, $builder, $data);
    $builder->addMany($fields);
    return $builder->build();
}

/**
 * Create form from factory
 * @param string $form
 * @param mixed $dataSource
 * @param array|null $provided
 * @return Form\Form
 */
function cms_create_form($form, $dataSource = null, array $provided = null)
{
    /** @var Form\AbstractFactory $factory */
    $factory = app($form);
    if ($provided !== null) {
        $factory->provideMany($provided);
    }
    return $factory->create($dataSource);
}

/**
 * Construct filter form in runtime
 * @param callable|null $callback
 * @param mixed $data
 * @return Form\Form
 */
function cms_construct_filter(callable $callback = null, $data = null)
{
    $builder = app(Form\BuilderInterface::class);
    if ($data !== null) {
        $builder->setDataSource($data);
    }

    $defaultFields = config('admin.filter_fields');
    // fix not translated labels
    foreach ($defaultFields as $fieldIdx => $defaultField) {
        if (isset($defaultField['label'])) {
            $defaultField['label'] = trans($defaultField['label']);
            $defaultFields[$fieldIdx] = $defaultField;
        }
    }
    $builder->addMany($defaultFields);

    if ($callback !== null) {
        $fields = call_user_func($callback, $builder, $data);
        $builder->addMany($fields);
    }

    return $builder->build();
}


//function cms_add_css($href)
//{
//    $asset = app(System\AssetInterface::class);
//    $hrefArr = (array)$href;
//    foreach ($hrefArr as $href) {
//        $asset->addCss($href);
//    }
//}
//
//function cms_add_js($src)
//{
//    $asset = app(System\AssetInterface::class);
//    $srcArr = (array)$src;
//    foreach ($srcArr as $src) {
//        $asset->addJs($src);
//    }
//}
//
//function cms_render_css()
//{
//    return app(System\AssetInterface::class)->renderCss();
//}
//
//function cms_render_js()
//{
//    return app(System\AssetInterface::class)->renderJs();
//}

/**
 * @param string $name
 * @param array|null $data
 * @private array|null $viewData
 * @return bool
 */
function cms_send_template($name, array $data = null, array $viewData = null)
{
    $template = app(EmailTemplate\TemplateRepository::class)->findByName($name);
    if ($template === null) {
        logger()->error('Email template not found', compact('name'));
        return false;
    }

    $mailer = app(EmailTemplate\MailerInterface::class);
    $mailer->setLogger(logger());

    return $mailer->send($template, $data, $viewData);
}

/**
 * @param string $page
 * @param Seo\SeoInterface $model
 * @param array $replacePairs
 * @return Seo\SeoManagerInterface
 */
function cms_seo($page = null, Seo\SeoInterface $model = null, array $replacePairs = [])
{
    $seo = app(Seo\SeoManagerInterface::class);
    if ($page !== null) {
        $seo->fromPage($page);
    }
    if ($model !== null) {
        $seo->fromModel($model);
    }
    if (count($replacePairs)) {
        $seo->replacePairs($replacePairs);
    }

    return $seo;
}

function cms_admin_upload_url($dir, $filename, $name = null)
{
    $file = rtrim(strtr(base64_encode($dir), '+/', '-_'), '=') . '.' .
        rtrim(strtr(base64_encode($filename), '+/', '-_'), '=') .
        ($name !== null ? '.' . rtrim(strtr(base64_encode($name), '+/', '-_'), '=') : null);
    return route('admin::upload_dl', $file);
}